from cleo.helpers import argument, option

from beegen.commands.smart.base import SmartBaseCommand
from beegen.commands.smart.core.chain import load_chain


class SmartRegexCommand(SmartBaseCommand):
    name = "smart regex"
    description = "Generate a regular expression based on the provided value."

    arguments = [argument("value", "The value to generate a regex for.")]
    options = [
        option(
            "language",
            "l",
            "The programming language to generate the example for.",
            flag=False,
        )
    ]

    def handle(self) -> int:
        self.line("")

        if not self.check_provider():
            return

        self.line_prefix(
            "Loading model(LLM) from provider "
            f"<comment>{self.provider.provider_name}</>"
        )

        value = self.argument("value")
        language = self.option("language") or "No Generate example"
        self.line_prefix("Generating a regex based on the provided value...")

        try:
            with self.console.status("") as _:
                template = self.__get_template_regex()
                chain = load_chain(template, self.provider.chat_model)
                response = chain.invoke({"value": value, "language": language})

            self.line_prefix("Model response:\n")
            self.print_markdown(response)
        except Exception:
            self.line_prefix("<error>An error occurred while generating the regex.</>")

        self.line("")

    def __get_template_regex(self) -> str:
        return """
        Create a regex based on the value provided below.
        Just create the regex, but if the user specifies the programming language,
        provide a simple example of how to use the regex in the chosen language.

        Value:
        {value}

        Language:
        {language}

        Reminder: Return only the regex and, if requested by the user, an example
        in the specified language. Both should be formatted using Markdown.
        The regex must be enclosed in `code blocks`, and the example code in the
        specified language should also be presented within its own `code block`.
        No comments or additional explanations should be provided.
        """



from beegen.commands.snippets.base import SnippetsBaseCommand


class SnippetsListCommand(SnippetsBaseCommand):
    name = "snippets list"
    description = "List all available snippets."

    def handle(self) -> int:
        self.line("")

        snippets = self.load_snippets()
        if not snippets.get("items"):
            self.line_prefix("No snippets found.")
            self.line("")
            return

        table = self.table()
        table.set_headers(["Snippet", "Description", "Type"])
        table.set_rows(
            [
                [
                    item,
                    value.get("description"),
                    "Code" if value.get("code") else "File",
                ]
                for item, value in snippets.get("items", {}).items()
            ]
        )
        table.render()
        self.line("")



import os

import streamlit as st

CURRENT_DIR = os.path.dirname(__file__)


def set_config():
    favicon_path = os.path.join(CURRENT_DIR, "images/favicon.ico")
    st.set_page_config(
        page_title="BeeGen - Chat",
        page_icon=favicon_path,
        layout="wide",
    )

    hide_decoration_bar_style = """
        <style>
            header {visibility: hidden;}
        </style>
    """
    st.markdown(hide_decoration_bar_style, unsafe_allow_html=True)


def set_page_header():
    image_path = os.path.join(CURRENT_DIR, "images/beegen.png")
    st.logo(image_path)

    with st.sidebar:
        st.image(image_path, use_column_width=True)
        st.subheader("BeeGen's Chat Interface")
        st.markdown(
            """
            BeeGen's chat was created to facilitate testing on any type of
            agent using API, thus allowing the use of any type of LLM or framework.
            """
        )
        st.divider()

        st.page_link("app.py", label="Settings", icon=":material/settings:")
        st.page_link("pages/playground.py", label="PlayGround", icon=":material/chat:")



import json

import uvicorn
from fastapi import Depends, FastAPI, HTTPException, Request
from fastapi.security import APIKeyHeader
from pydantic import ValidationError, create_model


class MockService:

    def __init__(self, mockfile: dict):
        self.__app = FastAPI()
        self.__api_key = None
        self.__api_key_header = APIKeyHeader(name="X-API-Key", auto_error=False)
        self.__mockfile = mockfile
        self.__create_service()

    def __create_service(self):

        if "authentication" in self.__mockfile:
            self.__api_key = self.__mockfile["authentication"].get("key", None)
            api_key_name = self.__mockfile["authentication"].get("name", "X-API-Key")
            self.__api_key_header = APIKeyHeader(name=api_key_name, auto_error=False)

        self.__app.title = self.__mockfile.get("name", "beegen")
        self.__app.description = self.__mockfile.get(
            "description", "BeeGen API mockfile"
        )
        self.__app.version = self.__mockfile.get("version", "0.1.0")

        for item in self.__mockfile.get("endpoints", []):
            self.__create_routes(item)

    def __create_routes(self, endpoint: dict):

        request_model = None
        if "request" in endpoint:
            request = endpoint.get("request", [])
            fields = {field["name"]: (field["type"], ...) for field in request}
            request_model = create_model("RequestModel", **fields)

        response = endpoint["response"].get("body", {})

        async def route(request: Request, key: str = Depends(self.__api_key_header)):
            check = all(
                [
                    access == "protected",
                    key != self.__api_key,
                    "authentication" in self.__mockfile,
                ]
            )
            if check:
                raise HTTPException(status_code=401, detail="Unauthorized")

            try:
                if request.method in ["POST", "PUT", "PATCH"]:
                    data = await request.json()
                    if request_model:
                        data = request_model(**data)
            except json.JSONDecodeError:
                raise HTTPException(status_code=400, detail="Invalid JSON body")
            except ValidationError as e:
                raise HTTPException(status_code=422, detail=e.errors())
            except Exception as e:
                raise HTTPException(status_code=500, detail=str(e))

            return response

        methods = {
            "GET": self.__app.get,
            "POST": self.__app.post,
            "PUT": self.__app.put,
            "PATCH": self.__app.patch,
            "DELETE": self.__app.delete,
        }

        path = endpoint.get("path")
        method = endpoint.get("method")
        status = endpoint["response"].get("status", 200)
        access = endpoint.get("access", "public")

        methods.get(method)(path, status_code=status)(route)

    def run(self):
        port = self.__mockfile.get("port", 8000)
        host = self.__mockfile.get("host", "0.0.0.0")
        uvicorn.run(self.__app, host=host, port=port)



import os
from typing import Any

from langchain_community.document_loaders import (
    DirectoryLoader,
    PyPDFDirectoryLoader,
    PyPDFLoader,
    TextLoader,
)
from langchain_community.vectorstores import FAISS
from langchain_text_splitters import RecursiveCharacterTextSplitter


class VectorStore:

    def __init__(self, source: str, file_type: str, embeddings):
        self.__source = source
        self.__documents = None
        self.__embeddings = embeddings
        self.__loader = self.__get_loader_method(file_type)

    def __get_loader_method(self, file_type: str):
        if file_type == "pdf":
            return self.__pdf_loader()
        return self.__text_loader(file_type)

    def __pdf_loader(self) -> Any:
        if os.path.isdir(self.__source):
            return PyPDFDirectoryLoader(self.__source)
        return PyPDFLoader(self.__source)

    def __text_loader(self, file_type: str) -> Any:
        if os.path.isdir(self.__source):
            extension = {"markdown": "md", "text": "txt"}.get(file_type, file_type)
            return DirectoryLoader(
                "../", glob=f"**/*.{extension}", loader_cls=TextLoader
            )
        return TextLoader(self.__source, encoding="utf-8")

    def load(self) -> None:
        self.__documents = self.__loader.load()

    def split_text(self) -> None:
        text_splitter = RecursiveCharacterTextSplitter(
            chunk_size=1000, chunk_overlap=200
        )
        self.__documents = text_splitter.split_documents(self.__documents)

    def size(self) -> int:
        return len(self.__documents)

    def save(self, folder_path: str) -> None:
        db = FAISS.from_documents(self.__documents, self.__embeddings)
        db.save_local(folder_path=folder_path)



from .anonymize_data import SmartAnonymizeCommand
from .pull_request_desc import SmartPullRequestDescriptionCommand
from .readme import SmartReadmeCommand
from .regex import SmartRegexCommand
from .translate import SmartTranslateCommand
from .vector_store import SmartCreateVectorStoreCommand

__all__ = [
    "SmartRegexCommand",
    "SmartReadmeCommand",
    "SmartCreateVectorStoreCommand",
    "SmartTranslateCommand",
    "SmartAnonymizeCommand",
    "SmartPullRequestDescriptionCommand",
]



import git
from cleo.helpers import option

from beegen.commands.smart.base import SmartBaseCommand
from beegen.commands.smart.core.chain import load_chain


class SmartPullRequestDescriptionCommand(SmartBaseCommand):
    name = "smart pull-request-desc"
    description = (
        "Generate a pull request description based on "
        "commit messages from the active branch."
    )

    options = [
        option(
            long_name="language",
            short_name="l",
            description="Define the language for the description translation.",
            flag=False,
        )
    ]

    def handle(self) -> int:
        self.line("")

        if not self.check_provider():
            return

        language = self.option("language") or "English"
        self.line_prefix(
            f"Generating pull request description into <info>{language}</>."
        )

        try:
            with self.console.status("") as _:
                template = self.__get_template_pr()
                chain = load_chain(template, self.provider.chat_model)
                commit_messages = self.__get_git_commits()
                response = chain.invoke(
                    {"commit_messages": commit_messages, "language": language}
                )

            self.line_prefix("Pull Request Description:\n")
            self.print_markdown(f"```\n{response}\n```")
        except Exception:
            self.line_prefix(
                "<error>An error occurred while generating the description.</>"
            )
            self.line_prefix(
                "<error>Please ensure you are in a valid Git repository.</>\n"
            )

        self.line("")

    def __get_template_pr(self) -> str:
        return """
        Given the following commit messages, generate a detailed
        pull request description in {language}.

        Commit messages:
        {commit_messages}

        Pull request description:
        """

    def __get_git_commits(self) -> str:
        repo = git.Repo(search_parent_directories=True)
        branch = repo.active_branch.name
        commits = list(
            repo.iter_commits(branch, max_count=10, no_merges=True, first_parent=True)
        )
        commit_messages = [commit.message.strip() for commit in commits]
        return "\n".join(commit_messages)



from typing import Any

from cleo.commands.command import Command
from rich.console import Console
from rich.markdown import Markdown
from rich.syntax import Syntax


class BaseCommand(Command):

    PREFIX = "> <fg=yellow>[beegen]</> "

    def __init__(self) -> None:
        super().__init__()
        self.console = Console()

    def line_prefix(self, text, style=None, verbosity=None):
        text = self.PREFIX + text if text else ""
        self.line(text)

    def ask_prefix(self, question: str, default: Any | None = None) -> Any:
        question = self.PREFIX + question if question else ""
        return self.ask(question, default)

    def print_markdown(self, text: str):
        markdown = Markdown(text)
        self.console.print(markdown)

    def print_code(self, text: str, language: str = "python"):
        syntax = Syntax(text, language, dedent=True)
        self.console.print(syntax)



import os

from cleo.helpers import argument, option

from beegen.commands.smart.base import SmartBaseCommand
from beegen.commands.smart.core.chain import load_chain


class SmartReadmeCommand(SmartBaseCommand):
    name = "smart readme"
    description = "Automatically generate a detailed README file for your project."

    arguments = [argument("project_path", description="The path to the project")]

    options = [
        option(
            long_name="language",
            short_name="l",
            description="Define the language for generating a readme file.",
            flag=False,
        )
    ]

    def handle(self) -> int:
        self.line("")

        if not self.check_provider():
            return

        project_path = self.argument("project_path")
        if not os.path.exists(project_path):
            self.line_prefix(f"Project path <error>'{project_path}'</> does not exist.")

        self.line(
            "<comment>"
            "Please provide one or more files with information or project "
            "configurations to improve the context.</>"
        )
        self.line(
            "<comment>Write the file names separated by commas "
            "(if there is more than one).</>\n"
        )
        self.line_prefix(
            "Loading model(LLM) from provider "
            f"<comment>{self.provider.provider_name}</>"
        )
        language = self.option("language") or "English"
        project_files = self.ask_prefix("Project files:", "")
        if not project_files:
            self.line_prefix("<error>No project files provided.</>")
            if self.confirm(
                f"{self.PREFIX}Do you like to continue without project files?", False
            ):
                self.__generate_readme(project_path, {}, language)
                return

            self.line("")
            return

        self.line_prefix("Loading project files...")
        context_files = self.__get_context_files(project_path, project_files)
        self.__generate_readme(project_path, context_files, language)

    def __generate_readme(self, path: str, context_files: dict, language: str):
        try:
            self.line_prefix("Generating README...")
            with self.console.status("") as _:
                project_details = self.__load_project(path)
                template = self.__get_template_readme()
                chain = load_chain(template, self.provider.chat_model)
                readme_text = chain.invoke(
                    {
                        "project_details": project_details,
                        "language": language,
                        "context_files": context_files,
                    }
                )
                self.__save_readme(readme_text)
        except Exception:
            self.line_prefix("<error>An error occurred while generating the readme.</>")
        finally:
            self.line("")

    def __get_context_files(self, path: str, files: str) -> str:
        context_files = ""
        list_files = files.split(",")
        for item in list_files:
            item = item.strip()
            file_path = os.path.join(path, item)
            if not os.path.exists(file_path):
                self.line_prefix(
                    f"<error>Project file '{file_path}' does not exist.</>"
                )
                return ""
            context_files += f"File: {item}\n{self.__load_file(file_path)}"
        return context_files

    def __load_file(self, file_path: str) -> str:
        with open(file_path, "r") as file:
            content = file.read()
            return content

    def __load_project(self, project_path: str) -> str:
        details = ""
        for dirpath, _, filenames in os.walk(project_path):
            check = any(["./." in dirpath, "__pycache__" in dirpath])
            if check:
                continue
            details += f"\nDirectory: {dirpath}\nFiles:\n"
            for filename in filenames:
                filepath = os.path.join(dirpath, filename)
                if self.__is_text_file(filepath):
                    details += f"- {filepath}\n"

        return details

    def __is_text_file(self, file_path, encoding="utf-8"):
        try:
            with open(file_path, "rb") as file:
                bytes = file.read(1024)
                bytes.decode(encoding)
            return True
        except (UnicodeDecodeError, TypeError):
            return False

    def __get_template_readme(self) -> str:
        return """
        You are a software developer with years of experience and need to create a
        README with relevant information about the project in Markdown. I will provide
        the directory structure, project file names, the language to be used, and the
        content of some files with relevant information or configurations about the
        project.

        List of directories and files:
        {project_details}

        Project files:
        {context_files}

        IMPORTANT:
        Try to generate the following topics in the "README":
        - Place a message in italics stating that this README was generated with the
        BeeGen tool.
        - Write a brief introduction.
        - Place some badges with the identified technologies just below the
        introduction text (without a title).
        - Perhaps a table with the main features of the project based on the provided
        files and directories (do not include the folder and file structure as
        it may be too large).
        - Prerequisites or dependencies according to the technology and the
        analyzed files.
        - Maybe instructions on how to use.
        - Perhaps a message for contribution.

        Language:
        {language}
        """

    def __save_readme(self, readme_text: str):
        filename = "README.md"
        if os.path.exists(filename):
            filename = "README_generated.md"

        with open(filename, "w") as file:
            file.write(readme_text)



from cleo.helpers import argument, option

from beegen.commands.smart.base import SmartBaseCommand
from beegen.commands.smart.core.vector_store import VectorStore


class SmartCreateVectorStoreCommand(SmartBaseCommand):
    name = "smart create-vectorstore"
    description = (
        "Create a local vector store using FAISS from specified files or directories."
    )

    arguments = [
        argument(
            name="path",
            description=(
                "The path to the file or directory to be included in the vector store."
            ),
        )
    ]

    options = [
        option(
            long_name="file-type",
            short_name="f",
            description="Specifies the type of the files (e.g., markdown, txt, pdf)",
            flag=False,
        )
    ]

    files_supported = ["html", "json", "markdown", "pdf", "text"]

    def handle(self) -> int:
        self.line("")

        if not self.check_provider():
            return

        file_type = self.option("file-type")
        if file_type is None:
            file_type = self.choice(
                f"{self.PREFIX}Please select the file type supported:",
                self.files_supported,
                0,
            )

        if file_type not in self.files_supported:
            self.line_prefix(f"<error>Unsupported file type: {file_type}</error>")
            self.line_prefix(f"Supported files: {', '.join(self.files_supported)}\n")
            return

        source_path = self.argument("path")
        self.__create_vector_store(source_path, file_type)

        self.line("")

    def __create_vector_store(self, source_path: str, file_type: str) -> None:
        try:
            self.line("")
            self.line_prefix(
                f"Loading document(s) from <comment>{source_path}</comment>"
            )

            with self.console.status("") as _:
                vectorstore = VectorStore(
                    source_path, file_type, self.provider.embeddings
                )
                vectorstore.load()

            self.line_prefix(
                f"Loaded <comment>{vectorstore.size()}</> document(s) from "
                f"<comment>{source_path}</>"
            )

            if self.confirm(
                "\nDo you want to split the documents into small chunks?", False
            ):
                vectorstore.split_text()

            self.line("")
            self.line_prefix(
                "Creating a vector store with the "
                f"<comment>{self.provider.embeddings_name}</> embeddings model "
                f"from provider <comment>{self.provider.provider_name}</>"
            )

            with self.console.status("") as _:
                vectorstore.save("./data")

            self.line_prefix(
                f"Vector store created and saved to <comment>{'./data'}</>"
            )
            self.__client_example(self.provider.embeddings_name)

        except Exception:
            self.line_prefix(
                "<error>An error occurred while creating the vector store.</>"
            )

    def __client_example(self, embedding_model: str):
        model = embedding_model if embedding_model else "mxbai-embed-large"
        example = f"""
        from langchain_community.embeddings import OllamaEmbeddings
        from langchain_community.vectorstores import FAISS

        # load embeddings model
        embeddings = OllamaEmbeddings(model="{model}")

        # load the vector store
        db = FAISS.load_local(
            folder_path="./data",
            embeddings=embeddings,
            allow_dangerous_deserialization=True
        )

        # query the vectorstore
        query = "Make a question"
        docs = db.similarity_search(query)
        print(docs[0].page_content)

        # convert vector store to retrieve
        retriever = db.as_retriever()
        docs = retriever.invoke(query)
        print(docs[0].page_content)
        """
        self.line_prefix("Use the following code as a starting point:\n")
        self.print_code(example)



from cleo.commands.command import Command
from pyfiglet import Figlet

from beegen.__version__ import __version__


class AboutCommand(Command):
    name = "about"
    description = "Shows information about BeeGen"

    def handle(self):
        custom_fig = Figlet(font="big")
        title = custom_fig.renderText("BeeGen")
        self.line(
            f"\n<fg=yellow>{title}</><fg=green>"
            "BeeGen is an intelligent command-line tool designed to assist developers\n"
            "with everyday tasks, leveraging the power of generative AI."
            "</>\n"
        )
        self.line(f"version: <fg=green>{__version__}</>\n")



import os
import sys

from streamlit.web import cli as stcli

from beegen.commands.base import BaseCommand


class UtilsChatCommand(BaseCommand):
    name = "utils chat"
    description = "BeeGen's chat interface for any language model"

    def handle(self) -> int:
        self.line("")

        current_dir = os.path.dirname(os.path.abspath(__file__))
        two_levels_up = os.path.dirname(os.path.dirname(current_dir))
        app_path = os.path.join(two_levels_up, "services", "chat", "app.py")

        sys.argv = [
            "streamlit",
            "run",
            app_path,
            "--theme.base",
            "dark",
            "--theme.primaryColor",
            "#dca927",
            "--theme.backgroundColor",
            "#080808",
            "--theme.secondaryBackgroundColor",
            "#161616",
            "--client.showSidebarNavigation",
            "false",
        ]
        sys.exit(stcli.main())



import os

from langchain_community.chat_models import ChatOllama
from langchain_community.embeddings import OllamaEmbeddings
from langchain_community.llms import Ollama
from langchain_google_genai import (
    ChatGoogleGenerativeAI,
    GoogleGenerativeAI,
    GoogleGenerativeAIEmbeddings,
)
from langchain_openai import ChatOpenAI, OpenAI, OpenAIEmbeddings


class Provider:
    def __init__(self, config: dict) -> None:
        self.__llm = None
        self.__chat_model = None
        self.__embeddings = None
        self.__embeddings_name = None
        self.__provider_name = None
        self.__temperature = 0
        self.__config = config
        self.__load_provider()

    @property
    def llm(self):
        """Returns the Large Language Model instance."""
        return self.__llm

    @property
    def chat_model(self):
        """Returns the Chat Large Language Model instance."""
        return self.__chat_model

    @property
    def embeddings(self):
        """Returns the Embeddings instance."""
        return self.__embeddings

    @property
    def embeddings_name(self):
        """Returns the embeddings name."""
        return self.__embeddings_name

    @property
    def provider_name(self):
        """Returns the provider name."""
        return self.__provider_name

    def __load_provider(self):
        self.__provider_name = self.__config["provider"]
        provider_method = self.__get_provider_method(self.__provider_name)
        if provider_method is None:
            self.line_prefix(
                f"Provider <error>{self.__provider_name}</> not supported."
            )

        # init provider model
        self.__llm, self.__chat_model, self.__embeddings = provider_method(
            self.__config
        )

    def __get_provider_method(self, provider_name: str):
        """Returns the provider method based on the given provider name."""
        return {
            "Ollama": self.__ollama_provider,
            "OpenAI": self.__openai_provider,
            "Google": self.__google_provider,
        }.get(provider_name, None)

    def __ollama_provider(self, parameters: dict):
        model_name = parameters.get("model_name", "llama3")
        self.__embeddings_name = parameters.get("embeddings_name", model_name)

        llm = Ollama(model=model_name)
        chat_model = ChatOllama(model=model_name)
        embeddings = OllamaEmbeddings(model=self.__embeddings_name)

        return llm, chat_model, embeddings

    def __openai_provider(self, parameters: dict):
        os.environ["OPENAI_API_KEY"] = parameters["api_key"]
        model_name = parameters.get("model_name", "gpt-3.5-turbo")
        self.__embeddings_name = parameters.get(
            "embeddings_name", "text-embedding-3-large"
        )

        llm = OpenAI(model_name=model_name, temperature=self.__temperature)
        chat_model = ChatOpenAI(model_name=model_name, temperature=self.__temperature)
        embeddings = OpenAIEmbeddings(model=self.__embeddings_name)

        return llm, chat_model, embeddings

    def __google_provider(self, parameters: dict):
        os.environ["GOOGLE_API_KEY"] = parameters["api_key"]
        model_name = parameters.get("model_name", "gemini-pro")
        self.__embeddings_name = parameters.get(
            "embeddings_name", "models/embedding-001"
        )

        llm = GoogleGenerativeAI(model=model_name, temperature=self.__temperature)
        chat_model = ChatGoogleGenerativeAI(
            model=model_name, temperature=self.__temperature
        )
        embeddings = GoogleGenerativeAIEmbeddings(model=self.__embeddings_name)

        return llm, chat_model, embeddings



import os

from cleo.helpers import argument, option

from beegen.commands.smart.base import SmartBaseCommand
from beegen.commands.smart.core.chain import load_chain


class SmartAnonymizeCommand(SmartBaseCommand):
    name = "smart anonymize-data"
    description = (
        "Anonymize sensitive data with generative AI to "
        "ensure privacy by masking identifiable information."
    )

    arguments = [argument("text", "The text to anonymize.", optional=True)]
    options = [option("file", "f", description="Define text file", flag=False)]

    def handle(self) -> int:
        self.line("")

        if not self.check_provider():
            return

        self.line_prefix(
            "Loading model(LLM) from provider "
            f"<comment>{self.provider.provider_name}</>"
        )

        text = self.argument("text")
        text_file = self.option("file")

        if not any([text, text_file]):
            self.line_prefix("Please provide a text or a text file for anonymization")
            self.line("")
            return

        try:
            if text:

                with self.console.status("") as _:
                    response = self.__anonymize_text(text)

                if not response:
                    self.line_prefix(
                        "<error>An error occurred while anonymizing data.</>"
                    )
                    self.line_prefix(
                        "<info>Please change the model or try again later.</>\n\n"
                    )
                    return
                self.line_prefix("Anonymized text:\n")
                self.print_markdown(f"```\n{response}\n```")

            if text_file:
                text_file_path = os.path.abspath(text_file)
                text_file_content = self.__read_text_file(text_file_path)
                if text_file_content:
                    with self.console.status("") as _:
                        content = self.__anonymize_text(text_file_content)
                    filename = f"anonymized_{text_file}"
                    self.__save_text_file(filename, content)
                    self.line_prefix(f"Anonymized text saved to {filename}")

            self.line("")

        except Exception:
            self.line_prefix("<error>An error occurred while anonymizing data.</>")

    def __anonymize_text(self, text: str) -> str:
        template = self.__get_template_anonymize()
        chain = load_chain(template, self.provider.chat_model)
        return chain.invoke({"text": text})

    def __read_text_file(self, file_path: str) -> str:
        if os.path.exists(file_path):
            with open(file_path, "r", encoding="utf-8") as file:
                return file.read()
        self.line_prefix("<error>Text file not found</>")
        return ""

    def __save_text_file(self, filename: str, content: str) -> bool:
        text_file_path = os.path.abspath(filename)
        with open(text_file_path, "w", encoding="utf-8") as file:
            file.write(content)

    def __get_template_anonymize(self) -> str:
        return """
        You are a data anonymization expert. Your task is to anonymize sensitive data
        such as names, emails, phone numbers, CPF, API keys, passwords, secret keys,
        or any type of sensitive information in the following text.
        Replace them with a placeholder (e.g., *** or [REDACTED]) while keeping the
        rest of the text intact.

        Original text: {text}

        Anonymized text:

        Reminder: Return only the anonymized text without any explanation.
        """



from beegen.commands.base import BaseCommand
from beegen.commands.smart.core.provider import Provider
from beegen.config import settings


class SmartBaseCommand(BaseCommand):
    def __init__(self) -> None:
        super().__init__()
        self.__config = settings.get_configurations()
        self.provider = self.__load_provider()

    def __load_provider(self) -> Provider:
        if self.__config.has_section("foundation_model"):
            try:
                provider_config = dict(self.__config["foundation_model"])
                return Provider(provider_config)
            except Exception:
                pass
        return None

    def check_provider(self) -> bool:
        if self.provider is None:
            self.line_prefix("<error>LLM provider configuration not found.</>")
            self.line_prefix(
                "Please run the command below to set up the required settings:\n"
            )
            self.print_markdown("```\nbeegen configure\n```")
            self.line("")

            self.line_prefix("You may also want to check your current configuration:\n")
            self.print_markdown("```\nbeegen configure --show\n```")
            self.line("")

            return False
        return True



import requests
import streamlit as st
from components import page_config
from requests import Response


def main():
    setup_page()
    chat_settings = st.session_state.get("chat_settings", {})

    if not chat_settings.get("endpoint"):
        st.warning("Endpoint not found. Please set endpoint in chat settings first")
        return

    if not chat_settings.get("request_body"):
        st.warning(
            "Request body not found. Please set request body in chat settings first"
        )
        return

    generate_chat()


def setup_page():
    page_config.set_config()
    page_config.set_page_header()
    st.header("PlayGround", divider="orange")


def generate_chat():
    if "messages" not in st.session_state:
        st.session_state["messages"] = [
            {
                "role": "assistant",
                "type": "text",
                "content": "How can I help you today?",
            }
        ]

    render_previous_messages()

    prompt = st.chat_input("Send a message")
    if not prompt:
        return

    st.session_state.messages.append(
        {"role": "user", "type": "text", "content": prompt}
    )
    st.chat_message("user").write(prompt)

    with st.spinner("Waiting for response..."):
        try:
            response = request_model(prompt)
            handle_response(response)
        except Exception as e:
            st.error(f"An error occurred in communication with the model: {e}")


def render_previous_messages():
    """Render previous chat messages from session state."""
    message_type = {"text": st.write, "code": st.code}
    for msg in st.session_state.messages:
        with st.chat_message(msg["role"]):
            content = msg["content"]
            message_type.get(msg["type"], st.write)(content)


def request_model(prompt: str) -> Response:
    """Send a request to the model with the given prompt."""
    headers = {"Content-Type": "application/json"}
    url = st.session_state.chat_settings.get("endpoint")
    data = st.session_state.chat_settings.get("request_body", "").replace(
        "{prompt}", prompt
    )

    return requests.post(url, headers=headers, data=data)


def handle_response(response: Response):
    """Handle the model's response and update chat messages."""

    if response.status_code != 200:
        raise Exception(response.content)

    full_json = st.session_state.chat_settings.get("full_json", True)
    response_type = st.session_state.chat_settings.get("response_type", "json")

    if response_type == "text":
        content = response.text
        add_message("assistant", "text", content)
        return

    content = response.json()
    if full_json:
        add_message("assistant", "code", content)
        return

    json_key = st.session_state.chat_settings.get("json_key", "")
    content = content.get(
        json_key,
        (
            f"The JSON key :orange[{json_key}] is not valid for the response.\n"
            "Please check the JSON key in settings and try again."
        ),
    )
    add_message("assistant", "text", content)


def add_message(role: str, msg_type: str, content: str):
    """Add a new message to the session state and display it."""
    st.session_state.messages.append(
        {"role": role, "type": msg_type, "content": content}
    )
    if msg_type == "text":
        st.chat_message(role).write(content)
    else:
        st.chat_message(role).code(content)


if __name__ == "__main__":
    main()



from .add import SnippetsAddCommand
from .list import SnippetsListCommand
from .remove import SnippetsRemoveCommand
from .use import SnippetsUseCommand

__all__ = [
    "SnippetsAddCommand",
    "SnippetsListCommand",
    "SnippetsRemoveCommand",
    "SnippetsUseCommand",
]



# -*- coding: utf-8 -*-

__author__ = "Adilson Pavan"

VERSION = (0, 1, 5)


def get_version():
    version = f"{VERSION[0]}.{VERSION[1]}"
    if VERSION[2]:
        version = f"{version}.{VERSION[2]}"
    return version


__version__ = get_version()



import json
import os
import shutil
import uuid
from typing import Tuple

from beegen.commands.base import BaseCommand
from beegen.config import settings


class SnippetsBaseCommand(BaseCommand):
    snippets_path = os.path.join(settings.CONFIG_ROOT_PATH, "snippets")
    snippets_filename = os.path.join(snippets_path, "snippets.json")
    snippets_files_path = os.path.join(snippets_path, "files")

    def __init__(self):
        super().__init__()
        if not os.path.exists(self.snippets_path):
            os.mkdir(self.snippets_path)
            os.mkdir(self.snippets_files_path)

    def load_snippets(self) -> list:
        """Load snippets from a JSON file."""
        snippets = {"items": {}}
        if os.path.exists(self.snippets_filename):
            try:
                with open(self.snippets_filename, "r", encoding="utf-8") as file:
                    return json.load(file)
            except (json.JSONDecodeError, IOError) as e:
                self.line_prefix(f"<error>Error loading snippets: {e}</>")
        return snippets

    def save_snippets(self, snippets: list) -> None:
        """Save snippets to a JSON file."""
        try:
            with open(self.snippets_filename, "w", encoding="utf-8") as file:
                json.dump(snippets, file, ensure_ascii=False, indent=4)
        except IOError as e:
            self.line_prefix(f"<error>Error saving snippets: {e}</>")

    def save_snippet_file(self, filename: str) -> Tuple[bool, str]:
        """Save snippet file."""
        if os.path.exists(filename):
            dest_filename = f"snippet-file-{str(uuid.uuid4())[:4]}.snippet"
            new_snippet_filename = os.path.join(self.snippets_files_path, dest_filename)
            shutil.copy(filename, new_snippet_filename)
            return True, dest_filename

        self.line_prefix("<error>Snippet file not found</>")
        return False

    def remove_snippet(self, name: str) -> None:
        """Remove snippet."""
        snippets = self.load_snippets()
        items = snippets["items"]

        if items[name].get("file"):
            snippet_file = os.path.join(self.snippets_files_path, items[name]["file"])
            os.remove(snippet_file)

        del snippets["items"][name]
        self.save_snippets(snippets)

    def exists_snippet(self, name: str) -> bool:
        """Check if snippet exists."""
        snippets = self.load_snippets()
        return name in snippets["items"]



import json

import streamlit as st
from streamlit_ace import st_ace

from beegen.services.chat.components import page_config

DEFAULT_CHAT_SETTINGS = {
    "endpoint": "",
    "request_body": "",
    "response_type": "",
    "response_type_index": 0,
    "full_json": False,
    "json_key": "",
}


def settings():
    setup_page()
    display_settings()


def setup_page():
    page_config.set_config()
    page_config.set_page_header()

    st.header("Settings", divider="orange")
    st.markdown("#### Configure the communication API with your model")


def display_settings():
    display_endpoint_setting()
    display_request_body_settings()
    display_response_settings()


def display_endpoint_setting():
    chat_settings = load_settings()

    with st.container(border=True):
        endpoint = st.text_input(
            "Endpoint",
            placeholder="Enter endpoint",
            value=chat_settings.get("endpoint"),
        )
        st.info(
            "The request to the endpoint will always be of type :orange[POST]",
            icon=":material/info:",
        )
        st.session_state.chat_settings["endpoint"] = endpoint


def display_request_body_settings():
    chat_settings = load_settings()

    with st.expander("Request Body"):
        st.markdown(
            """
            The request body will be sent as a JSON object.\n
            Use the variable :orange[{prompt}] to insert the text typed by the user
            in the chat, for example:
            """
        )
        st.code(
            """
            {
                "model": "llama3",
                "prompt": "{prompt}",
                "stream": false
            }
            """,
            language="json",
        )
        st.write("Define the request body:")
        request_body = st_ace(
            language="json",
            keybinding="vscode",
            min_lines=10,
            max_lines=None,
            font_size=14,
            tab_size=4,
            wrap=False,
            show_gutter=True,
            show_print_margin=False,
            readonly=False,
            annotations=None,
            theme="chaos",
            value=chat_settings.get("request_body"),
        )
        st.session_state.chat_settings["request_body"] = request_body
        if not is_valid_json(request_body):
            st.session_state.chat_settings["request_body"] = ""
            st.error("The JSON provided for the request body is invalid")


def display_response_settings():
    chat_settings = load_settings()

    with st.expander("Response"):
        options = ["json", "text"]
        response_type = st.radio(
            "Select your response type",
            options,
            captions=[
                "The response is a json object.",
                "The response is a text in a string.",
            ],
            index=chat_settings.get("response_type_index", 0),
        )
        response_type_index = options.index(response_type)

        json_key = ""
        full_json = False

        if response_type == "json":
            full_json = st.checkbox(
                "Check to return the complete JSON in the chat",
                value=chat_settings.get("full_json", False),
            )
            if not full_json:
                json_key = st.text_input(
                    "JSON Key",
                    placeholder="Set the key from which the value will be returned",
                    value=chat_settings.get("json_key", ""),
                )
        st.session_state.chat_settings.update(
            {
                "response_type": response_type,
                "response_type_index": response_type_index,
                "full_json": full_json,
                "json_key": json_key,
            }
        )


def load_settings() -> dict:
    if "chat_settings" not in st.session_state:
        st.session_state["chat_settings"] = DEFAULT_CHAT_SETTINGS.copy()
    return st.session_state.chat_settings


def is_valid_json(json_string: str) -> bool:
    try:
        if json_string:
            json.loads(json_string)
        return True
    except json.JSONDecodeError:
        return False



import os
from configparser import ConfigParser

CONFIG_FILENAME = "config.ini"
CONFIG_ROOT_PATH = os.path.join(os.path.expanduser("~"), ".beegen")


def config_path_exists() -> bool:
    return os.path.exists(CONFIG_ROOT_PATH)


def config_file_exists() -> bool:
    filename = os.path.join(CONFIG_ROOT_PATH, CONFIG_FILENAME)
    return os.path.exists(filename)


def get_configurations() -> ConfigParser:
    configuration = ConfigParser()
    if config_file_exists():
        configuration.read(os.path.join(CONFIG_ROOT_PATH, CONFIG_FILENAME))
    return configuration


def set_configurations(configuration: ConfigParser):
    filename = os.path.join(CONFIG_ROOT_PATH, CONFIG_FILENAME)
    with open(filename, "w") as config_file:
        configuration.write(config_file)



import secrets

import yaml
from cleo.helpers import argument

from beegen.commands.base import BaseCommand


class MockCreateCommand(BaseCommand):
    name = "mock create"
    description = "Generate a mockfile to configure the mock API."

    arguments = [
        argument("filename", description="Define the mock filename", optional=True)
    ]

    def handle(self) -> int:
        self.line("")

        filename = "mockfile.yml"
        if self.argument("filename"):
            filename = self.argument("filename")

        self.__generate_mockfile(filename)

    def __generate_mockfile(self, filename):
        self.line_prefix("Generating mockfile...")

        mockfile = self.__get_mockfile()
        with open(filename, "w") as f:
            yaml.dump(mockfile, f)

        self.line_prefix(f"Mockfile <comment>{filename}</comment> generated!")
        self.line("")

    def __get_mockfile(self) -> dict:
        return {
            "version": "0.1.0",
            "name": "BeeGen API",
            "host": "0.0.0.0",
            "port": 8000,
            "description": "BeeGen mockfile API example",
            "authentication": {
                "type": "api_key",
                "key": secrets.token_urlsafe(32),
                "name": "X-API-Key",
            },
            "endpoints": [
                {
                    "method": "GET",
                    "path": "/users",
                    "access": "public",
                    "response": {
                        "status": 200,
                        "body": [
                            {
                                "id": 1,
                                "name": "John Doe",
                                "email": "john.doe@example.com",
                            },
                            {
                                "id": 2,
                                "name": "Jane Doe",
                                "email": "jane.doe@example.com",
                            },
                        ],
                    },
                },
                {
                    "method": "POST",
                    "path": "/user",
                    "access": "protected",
                    "request": [
                        {
                            "name": "id",
                            "type": "int",
                            "description": "The user ID",
                        },
                        {
                            "name": "name",
                            "type": "str",
                            "description": "The user name",
                        },
                        {
                            "name": "email",
                            "type": "str",
                            "description": "The user email",
                        },
                    ],
                    "response": {"status": 201, "body": {"status": "created"}},
                },
            ],
        }



from cleo.helpers import argument

from beegen.commands.snippets.base import SnippetsBaseCommand


class SnippetsRemoveCommand(SnippetsBaseCommand):
    name = "snippets remove"
    description = "Remove a snippet by its name."

    arguments = [argument("name", description="Set the snippet name")]

    def handle(self) -> int:
        self.line("")
        name = self.argument("name")
        if self.exists_snippet(name):
            if self.confirm("Do you want to remove this snippet?"):
                self.line("")
                self.remove_snippet(name)
                self.line_prefix(f"Snippet <comment>{name}</> removed")
        else:
            self.line_prefix(f"Snippet <error>{name}</> not found")
        self.line("")



import os
import shutil

from cleo.helpers import argument

from beegen.commands.snippets.base import SnippetsBaseCommand


class SnippetsUseCommand(SnippetsBaseCommand):
    name = "snippets use"
    description = "Use the specified snippet."

    arguments = [argument("name", description="Set the snippet name")]

    def handle(self) -> int:
        self.line("")
        name = self.argument("name")
        if self.exists_snippet(name):
            snippet = self.__get_snippet(name)
            if snippet.get("code"):
                self.line_prefix("Code:\n")
                code = f"```\n{snippet.get('code')}\n```"
                self.print_markdown(code)
            else:
                self.line_prefix(
                    "The snippet is of type file, please specify the output file."
                )
                filename = self.ask_prefix("Output file:", "snippet.out")
                self.__save_output_file(filename, snippet)
                self.line_prefix(f"Snippet saved to <info>{filename}</>")
        else:
            self.line_prefix(f"Snippet <error>{name}</> not found")
        self.line("")

    def __get_snippet(self, name: str) -> dict:
        snippets = self.load_snippets()
        return snippets["items"][name]

    def __save_output_file(self, filename: str, snippet: dict):
        snippet_file = os.path.join(self.snippets_files_path, snippet.get("file"))
        snippet_output = os.path.join(os.path.curdir, filename)
        shutil.copy(snippet_file, snippet_output)



from configparser import ConfigParser

from cleo.helpers import option

from beegen.config import settings

from .base import BaseCommand


class ConfigureCommand(BaseCommand):
    name = "configure"
    description = "Configure the LLM and access keys for usage."

    options = [
        option(
            long_name="show",
            short_name="s",
            description="Show the current configuration.",
            flag=True,
        )
    ]

    def handle(self) -> int:
        self.line("")

        show_config = self.option("show")
        if show_config:
            self.__show_config()
            return

        self.__save_config()

    def __show_config(self) -> None:
        config = settings.get_configurations()
        if config.has_section("foundation_model"):
            provider_config = dict(config["foundation_model"])
            self.line_prefix("Current configuration:\n")
            self.line_prefix(f"<question>Provider:</> {provider_config['provider']}")
            self.line_prefix(
                f"<question>Model name:</> {provider_config['model_name']}"
            )
            self.line_prefix(
                f"<question>Embeddings name:</> {provider_config['embeddings_name']}\n"
            )
            return

        self.line_prefix("<error>LLM provider configuration not found.</>")
        self.line("")

    def __save_config(self) -> None:
        provider = self.choice(
            f"{self.PREFIX}Please select your favorite LLM provider",
            ["Ollama", "OpenAI", "Google"],
            0,
        )

        self.line("")
        model_name = self.ask_prefix("Model name:", "llama3")
        embeddings_name = self.ask_prefix("Embeddings model name:", "mxbai-embed-large")

        api_key = ""
        if provider != "Ollama":
            api_key = self.ask_prefix("API key:", "")

        self.line("")
        config = ConfigParser()
        config["foundation_model"] = {
            "provider": provider,
            "api_key": api_key,
            "model_name": model_name,
            "embeddings_name": embeddings_name,
        }
        settings.set_configurations(config)



from langchain_core.output_parsers import StrOutputParser
from langchain_core.prompts import ChatPromptTemplate
from langchain_core.runnables.base import RunnableSequence


def load_chain(template: str, model) -> RunnableSequence:
    prompt = ChatPromptTemplate.from_template(template)
    chain = prompt | model | StrOutputParser()
    return chain



import os

from cleo.application import Application as BaseApplication

from .commands.about import AboutCommand
from .commands.configure import ConfigureCommand
from .commands.mock import MockCreateCommand, MockRunCommand
from .commands.smart import (
    SmartAnonymizeCommand,
    SmartCreateVectorStoreCommand,
    SmartPullRequestDescriptionCommand,
    SmartReadmeCommand,
    SmartRegexCommand,
    SmartTranslateCommand,
)
from .commands.snippets import (
    SnippetsAddCommand,
    SnippetsListCommand,
    SnippetsRemoveCommand,
    SnippetsUseCommand,
)
from .commands.utils import UtilsChatCommand
from .config import settings

try:
    from beegen.__version__ import __version__
except ImportError:
    from __version__ import __version__


class Application(BaseApplication):
    def __init__(self):
        super(Application, self).__init__("beegen", __version__)

        # create case not exists
        if not settings.config_path_exists():
            os.mkdir(settings.CONFIG_ROOT_PATH)

        # add commands
        for command in self.get_default_commands():
            self.add(command)

    def get_default_commands(self) -> list:
        commands = [
            AboutCommand(),
            MockCreateCommand(),
            MockRunCommand(),
            SnippetsAddCommand(),
            SnippetsListCommand(),
            SnippetsRemoveCommand(),
            SnippetsUseCommand(),
            ConfigureCommand(),
            SmartAnonymizeCommand(),
            SmartRegexCommand(),
            SmartReadmeCommand(),
            SmartCreateVectorStoreCommand(),
            SmartTranslateCommand(),
            UtilsChatCommand(),
            SmartPullRequestDescriptionCommand(),
        ]
        return commands



import json
import os
import pathlib

import yaml
from cleo.helpers import argument
from jsonschema import validate

from beegen.commands.base import BaseCommand
from beegen.services.mock import MockService


class MockRunCommand(BaseCommand):
    name = "mock run"
    description = "Run the mock API server using the mockfile."

    arguments = [
        argument(
            "filename",
            "Path to the yaml file containing the mock API configurations.",
            optional=True,
        )
    ]

    def handle(self) -> int:
        self.line("")

        filename = "mockfile.yml"
        if self.argument("filename"):
            filename = self.argument("filename")

        if os.path.exists(filename) is False:
            self.line_prefix(f"Mockfile <error>{filename}</error> not found!")
            self.line_prefix("Please create a mockfile first.")
            self.line("")
            return

        schema = self.__read_schema()
        mockfile = self.__read_mockfile(filename)

        self.line_prefix("Validating mockfile...")
        validate(mockfile, schema)

        self.line_prefix("Starting mock API server...")
        self.line("")

        service = MockService(mockfile)
        service.run()

    def __read_mockfile(self, filename: str) -> dict:
        self.line_prefix("Reading mockfile...")

        with open(filename, "r") as f:
            mockfile = yaml.safe_load(f)

        self.line_prefix(f"Mockfile <comment>{filename}</comment> read!")

        return mockfile

    def __read_schema(self) -> dict:
        current_path = pathlib.Path(__file__).parent.resolve()
        filename = f"{current_path}/schema.json"

        with open(filename, "r") as f:
            schema = json.load(f)

        return schema



from cleo.helpers import argument, option

from beegen.commands.smart.base import SmartBaseCommand
from beegen.commands.smart.core.chain import load_chain


class SmartTranslateCommand(SmartBaseCommand):
    name = "smart translate"
    description = "Translate a given text into the specified language."

    arguments = [argument("text", "The value to translate.")]

    options = [
        option(
            long_name="language",
            short_name="l",
            description="Set the language for text translation.",
            flag=False,
        )
    ]

    def handle(self) -> int:
        self.line("")

        if not self.check_provider():
            return

        text = self.argument("text")
        language = self.option("language") or "English"
        self.line_prefix(f"Translating text into <info>{language}</>.")

        try:
            with self.console.status("") as _:
                template = self.__get_template_translate()
                chain = load_chain(template, self.provider.chat_model)
                response = chain.invoke({"text": text, "language": language})

            self.line_prefix("Translation:\n")
            self.print_markdown(f"```\n{response}\n```")
        except Exception:
            self.line_prefix("<error>An error occurred while translating the text.</>")

        self.line("")

    def __get_template_translate(self) -> str:
        return """
        Translate the text into the language specified below:

        Text:
        {text}

        Language:
        {language}

        Reminder: Return only the translated text.
        """



import os
import uuid

from cleo.helpers import argument, option

from beegen.commands.snippets.base import SnippetsBaseCommand


class SnippetsAddCommand(SnippetsBaseCommand):
    name = "snippets add"
    description = "Add a new code snippet."

    arguments = [argument("name", description="Define the snippet name", optional=True)]
    options = [
        option(
            "code",
            "c",
            description="Define snippet code",
            flag=False,
        ),
        option(
            "file",
            "f",
            description="Define snippet file",
            flag=False,
        ),
    ]

    def handle(self) -> int:
        self.line("")

        name = self.argument("name")
        if not name:
            name = self.ask_prefix("Snippet name:", f"snippet-{str(uuid.uuid4())[:4]}")

        description = self.ask_prefix(
            "Description:", "Description default of the snippet"
        )

        snippet_code = self.option("code")
        snippet_file = self.option("file")

        if not any([snippet_code, snippet_file]):
            snippet_code = self.ask_prefix("Snippet code:")

        if snippet_file:
            snippet_file_path = os.path.abspath(snippet_file)
            saved, snippet_file = self.save_snippet_file(snippet_file_path)
            if not saved:
                self.line_prefix(
                    "<error>Unable to save the file, check if the path is correct</>"
                )
                return

        snippet = {
            "description": description,
            "code": snippet_code,
            "file": snippet_file,
        }

        self.__add_snippet(name, snippet)
        self.line("")

    def __add_snippet(self, name: str, snippet: dict) -> None:
        snippets = self.load_snippets()

        if self.exists_snippet(name):
            self.remove_snippet(name)

        snippets["items"][name] = snippet
        self.save_snippets(snippets)
        self.line_prefix(f"Snippet <comment>{name}</> added")
